@@ -5,7 +5,6 @@ use crate::util::{
     event::{Event, Events},
     iter::IteratorExt
 };
-use gumdrop::Options;
 use freebsd_libgeom::{Snapshot, Statistics, Tree};
 use nix::time::{ClockId, clock_gettime};
 use regex::Regex;
@@ -17,9 +16,9 @@ use std::{
     mem,
     num::NonZeroU16,
     ops::BitOrAssign,
-    str::FromStr,
     time::Duration
 };
+use structopt::StructOpt;
 use termion::{
     event::Key,
     input::MouseTerminal,
@@ -65,56 +64,63 @@ fn popup_layout(x: u16, y: u16, r: Rect) -> Rect {
 }
 
 /// Drop-in replacement for gstat(8)
-// TODO: shorten the help options so they fit on 80 columns.
-#[derive(Debug, Default, Deserialize, Options, Serialize)]
+#[derive(Debug, Default, Deserialize, Serialize, StructOpt)]
 struct Cli {
-    #[options(help = "print help message")]
-    help: bool,
-    /// only display providers that are at least 0.1% busy
-    #[options(short = 'a')]
+    /// Only display providers that are at least 0.1% busy
+    #[structopt(short = "a", long = "auto")]
     auto: bool,
-    /// display statistics for delete (BIO_DELETE) operations.
-    #[options(short = 'd')]
+    /// Display statistics for delete (BIO_DELETE) operations.
+    #[structopt(short = "d", long = "delete")]
     delete: bool,
-    /// only display devices with names matching filter, as a regex.
-    #[options(short = 'f')]
+    /// Only display devices with names matching filter, as a regex.
+    #[structopt(short = "f", long = "filter")]
     filter: Option<String>,
-    /// display statistics for other (BIO_FLUSH) operations.
-    #[options(short = 'o')]
+    /// Display statistics for other (BIO_FLUSH) operations.
+    #[structopt(short = "o", long = "other")]
     other: bool,
-    /// display block size statistics
-    #[options(short = 's')]
+    /// Display block size statistics
+    #[structopt(short = "s", long = "size")]
     size: bool,
-    /// display update interval, in microseconds or with the specified unit
-    #[options(short = 'I')]
-    // Note: argh has a "from_str_fn" property that could be used to create a
-    // custom parser, to parse interval directly to an int or a Duration.  That
-    // would make it easier to save the config file.  But gumpdrop doesn't have
-    // that option.
-    interval: Option<String>,
-    /// only display physical providers (those with rank of 1).
-    #[options(short = 'p')]
+    /// Only display physical providers (those with rank of 1).
+    #[structopt(short = "p", long = "physical")]
     physical: bool,
-    /// Reset the config file
+    /// Reset the config file to defaults
     #[serde(skip)]
+    #[structopt(long = "reset-config")]
     reset_config: bool,
     /// Reverse the sort
-    #[options(short = 'r')]
+    #[structopt(short = "r", long = "reverse")]
     reverse: bool,
     /// Sort by the named column.  The name should match the column header.
+    #[structopt(short = "S", long = "sort")]
     sort: Option<String>,
     /// Bitfield of columns to enable
-    // TODO: hide this from the CLI, either using
-    // https://github.com/murarth/gumdrop/issues/52
-    // or by switching to structopt
     #[serde(default = "default_columns_enabled")]
+    #[structopt(skip)]
     columns: Option<ColumnsEnabled>,
+    /// Display update interval, in microseconds or with the specified unit
+    #[structopt(
+        short = "I",
+        long = "interval",
+        parse(try_from_str = Cli::duration_from_str)
+    )]
+    interval: Option<Duration>,
+}
+
+impl Cli {
+    fn duration_from_str(s: &str) -> Result<Duration, humanize_rs::ParseError> {
+        if let Ok(us) = s.parse::<u64>() {
+            // With no units, default to microseconds
+            Ok(Duration::from_micros(us))
+        } else {
+            humanize_rs::duration::parse(s)
+        }
+    }
 }
 
 impl BitOrAssign for Cli {
     #[allow(clippy::or_fun_call)]
     fn bitor_assign(&mut self, rhs: Self) {
-        self.help |= rhs.help;
         self.auto |= rhs.auto;
         self.delete |= rhs.delete;
         self.filter = rhs.filter.or(self.filter.take());
@@ -189,14 +195,6 @@ fn default_columns_enabled() -> Option<ColumnsEnabled> {
     Some(Default::default())
 }
 
-// TODO: remove this impl.  It only exists because gumdrop can't skip a field.
-impl FromStr for ColumnsEnabled {
-    type Err = io::Error;
-    fn from_str(_s: &str) -> Result<Self, Self::Err> {
-        Ok(Self::default())
-    }
-}
-
 struct Columns {
     cols: [Column; Columns::LEN],
     state: ListState
@@ -506,8 +504,8 @@ impl DataSource {
             if let Some(gident) = self.tree.lookup(curstat.id()) {
                 if let Some(rank) = gident.rank() {
                     let stats = Statistics::compute(curstat, prevstat, etime);
-                    let elem = Element::new(&gident.name().to_string_lossy(),
-                        rank, &stats);
+                    let name = gident.name().unwrap().to_string_lossy();
+                    let elem = Element::new(&name, rank, &stats);
                     self.items.push(elem);
                 }
             }
@@ -580,25 +578,19 @@ impl StatefulTable {
     }
 }
 
+// https://github.com/rust-lang/rust-clippy/issues/7483
+#[allow(clippy::or_fun_call)]
 fn main() -> Result<(), Box<dyn Error>> {
-    let cli: Cli = Cli::parse_args_default_or_exit();
-    let mut cfg: Cli = confy::load("gstat-rs")?;
-    if cli.reset_config {
-        cfg = cli;
+    let cli: Cli = Cli::from_args();
+    let mut cfg = if cli.reset_config {
+        cli
     } else {
+        let mut cfg: Cli = confy::load("gstat-rs")?;
         cfg |= cli;
-    }
-    let mut filter = cfg.filter.as_ref().map(|s| Regex::new(s).unwrap());
-    let mut tick_rate: Duration = match cfg.interval.as_mut() {
-        None => Duration::from_secs(1),
-        Some(s) => {
-            if s.parse::<i32>().is_ok() {
-                // Add the default units
-                s.push_str("us");
-            }
-            humanize_rs::duration::parse(s)?
-        }
+        cfg
     };
+    let mut filter = cfg.filter.as_ref().map(|s| Regex::new(s).unwrap());
+    let mut tick_rate = cfg.interval.unwrap_or(Duration::from_secs(1));
     let mut editting_regex = false;
     let mut new_regex = String::new();
     let mut paused = false;
@@ -650,10 +642,25 @@ fn main() -> Result<(), Box<dyn Error>> {
                 .filter(|col| col.enabled)
                 .map(|col| col.width)
                 .collect::<Vec<_>>();
+            let max_name_width = data.items.iter()
+                .filter(|elem| !cfg.auto || elem.pct_busy > 0.1)
+                .filter(|elem| !cfg.physical || elem.rank == 1)
+                .filter(|elem|
+                        filter.as_ref()
+                        .map(|f| f.is_match(&elem.name))
+                        .unwrap_or(true)
+                ).map(|elem| elem.name.len() as u16)
+                .max()
+                .unwrap_or(0);
             let twidth: u16 = columns.cols.iter()
                 .filter(|col| col.enabled)
-                .map(|col| col.min_width())
-                .sum();
+                .map(|col| {
+                    if col.name == "Name" {
+                        max_name_width
+                    } else {
+                        col.min_width()
+                    }
+                }).sum();
             let ntables = NonZeroU16::new(f.size().width / twidth)
                 .unwrap_or_else(|| NonZeroU16::new(1).unwrap());
             let rects = Layout::default()
@@ -670,9 +677,8 @@ fn main() -> Result<(), Box<dyn Error>> {
                         filter.as_ref()
                         .map(|f| f.is_match(&elem.name))
                         .unwrap_or(true)
-                ).map(|elem| {
-                    elem.row(&columns)
-                }).deinterleave::<Vec<_>>(ntables.into());
+                ).map(|elem| elem.row(&columns))
+                .deinterleave::<Vec<_>>(ntables.into());
             for (i, rows) in multirows.into_iter().enumerate() {
                 let t = table.table(header.clone(), rows, &widths);
                 f.render_stateful_widget(t, rects[i], &mut table.state);
@@ -821,13 +827,11 @@ fn main() -> Result<(), Box<dyn Error>> {
                         }
                         Key::Char('<') => {
                             tick_rate /= 2;
-                            let s = tick_rate.as_micros().to_string();
-                            cfg.interval = Some(s);
+                            cfg.interval = Some(tick_rate);
                         }
                         Key::Char('>') => {
                             tick_rate *= 2;
-                            let s = tick_rate.as_micros().to_string();
-                            cfg.interval = Some(s);
+                            cfg.interval = Some(tick_rate);
                         }
                         Key::Char('F') => {
                             cfg.filter = None;
@@ -857,9 +861,9 @@ fn main() -> Result<(), Box<dyn Error>> {
                             table.previous();
                         }
                         Key::Delete => {
-                            // TODO: persist this change, and make it reversible.
-                            if let Some(idx) = sort_idx {
-                                columns.cols[idx].enabled = false;
+                            if let Some(i) = sort_idx {
+                                cfg.columns.as_mut().unwrap().0 ^= 1 << i;
+                                columns.cols[i].enabled ^= true;
                             }
                         }
                         Key::Insert => {
